//
// Sample DirectShow Sink Filter that accepts data for use in application
//
// Copyright Geraint Davies January 2000
//

#include "stdafx.h"

#include "filter.h"

// --- pin methods ------

CSinkInput::CSinkInput(CAppSink* pFilter, CCritSec*pLock, HRESULT*phr)
: CBaseInputPin(
    NAME("App-Direct sink pin"),
    pFilter,
    pLock,
    phr,
    L"Input"),
    m_pFilter(pFilter)
{
}

// check if the pin can support this specific proposed type and format
HRESULT 
CSinkInput::CheckMediaType(const CMediaType *pmt) 
{
    // just for example, we accept a common set of
    // video types
    if (*pmt->Type() != MEDIATYPE_Video) {
	return S_FALSE;
    }
    if ((*pmt->FormatType() != FORMAT_VideoInfo) &&
	(*pmt->FormatType() != FORMAT_VideoInfo2)) {
	return S_FALSE;
    }

    if ((*pmt->Subtype() != MEDIASUBTYPE_YUY2) &&
        (*pmt->Subtype() != MEDIASUBTYPE_RGB565) &&
	(*pmt->Subtype() != MEDIASUBTYPE_RGB24) ) {
	return S_FALSE;
    }
    return S_OK;
}

// these methods on the input pin just forward calls to
// the app
STDMETHODIMP 
CSinkInput::Receive(IMediaSample *pSample)
{
    return m_pFilter->Receive(pSample);
}

STDMETHODIMP 
CSinkInput::EndOfStream(void)
{
    return m_pFilter->EndOfStream();
}

// --- filter methods ---
CAppSink::CAppSink(LPUNKNOWN pUnkOuter)
: CBaseFilter(
    NAME("App-Direct Sink filter"),
    pUnkOuter,
    &m_Lock,
    GUID_NULL),
  m_pInput(NULL)
{
}

CAppSink::~CAppSink()
{
    delete m_pInput;
}


// these are the methods you need to override
// to handle the data and end-of-stream calls
HRESULT 
CAppSink::Receive(IMediaSample* pSample)
{
    return S_OK;
}

// called from pin -- override if you care about EOS
HRESULT 
CAppSink::EndOfStream()
{
    // all data has now arrived
    // delay this call if the data is queued in your app somewhere
    NotifyEvent(EC_COMPLETE, 0, 0);

    return S_OK;
}

// --- graph building (examples) --------- 

CAppSinkBuilder::CAppSinkBuilder()
: m_pSink(NULL),
  m_pGraph(NULL)
{
    CoInitialize(NULL);
}

CAppSinkBuilder::~CAppSinkBuilder()
{
    if (m_pSink) {
	m_pSink->Release();
    }
    if (m_pGraph) {
	m_pGraph->Release();
    }
    CoUninitialize();
}


HRESULT 
CAppSinkBuilder::BuildFromFile(LPCWSTR pszFile)
{
    // make a filter graph
    HRESULT hr = CoCreateInstance(
			CLSID_FilterGraph,
			NULL,
			CLSCTX_INPROC,
			IID_IGraphBuilder,
			(void**)&m_pGraph);

    if (FAILED(hr)) {
	return hr;
    }

    //insert the sink filter into the graph first
    MakeSink();

    // now render the file. The sink filter will
    // be used when possible (and it only accepts uncompressed
    // video types)
    return m_pGraph->RenderFile(pszFile, NULL);
}

HRESULT 
CAppSinkBuilder::Run()
{
    IMediaControl* pControl;
    HRESULT hr = m_pGraph->QueryInterface(IID_IMediaControl, (void**)&pControl);
    if (SUCCEEDED(hr)) {
	hr = pControl->Run();
	pControl->Release();
    }
    return hr;
}

void 
CAppSinkBuilder::MakeSink()
{
    if (m_pSink) {
	m_pSink->Release();
	m_pSink = NULL;
    }

    m_pSink = new CAppSink(NULL);
    // make the initial refcount 1 to match COM creation
    m_pSink->AddRef();

    // add to graph -- nb need to Query properly for the
    // right interface before giving that to the graph object
    IBaseFilter* pSinkFilter;
    HRESULT hr = m_pSink->QueryInterface(IID_IBaseFilter, (void**)&pSinkFilter);
    if (SUCCEEDED(hr)) {
	hr = m_pGraph->AddFilter(pSinkFilter, L"App-Direct Sink");
	pSinkFilter->Release();	    // corresponds to QueryInterface
    }
}
